/* Automatically generated by po2tbl.sed from TUA.pot.  */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "libgettext.h"

const struct _msg_ent _msg_tbl[] = {
  {"", 1},
  {"Wrong number of Parameters: %s needs just one UserName", 2},
  {"Error in processing %s\n", 3},
  {"Bad -f argument. Error factor fixed to %.1f\n", 4},
  {"UNKNOWN", 5},
  {"\
There is an error in %s...\n\
\t%s\n", 6},
  {"\
\n\
Countries I know about:\n", 7},
  {"Cannot parse pcd file '%s'. Ignoring.\n", 8},
  {"Country '%s' not found. I will omit phone costs.\n", 9},
  {"\
%s UUCP Analysis for ``%s'',\n\
  since %s to %s (%d day(s))\n\
\n", 10},
  {"whistory: probably you need uucp's write permission", 11},
  {"\
%s UUCP Analysis for ``%s''\n\
\n", 12},
  {"Wrong number of Parameters: %s needs a LongOption name", 13},
  {"%s: option '%s' not found!", 14},
  {"More> ", 15},
  {"TclError", 16},
  {"TclError %d", 17},
  {"overlapping slots: ", 18},
  {"{ rate %d, %s at line %d } and ", 19},
  {"{ rate %d, %s at line %d }\n", 20},
  {"overlapping costs!", 21},
  {"Rate symbol ", 22},
  {"%s not yet defined!\n", 23},
  {"error: symbol %.30s... too long!\n", 24},
  {"Wrong number of Parameters: %s needs an PortName and its Alias", 25},
  {"Wrong number of Parameters: %s needs an PortName and its GlobAlias", 26},
  {"Wrong number of Parameters: %s needs just one PortName", 27},
  {"Wrong number of Parameters: %s needs an SystemName and its Alias", 28},
  {"Wrong number of Parameters: %s needs an SystemName and its GlobAlias", 29},
  {"Wrong number of Parameters: %s needs just one SystemName", 30},
  {"\
\tCommands:\n\
\t=========", 31},
  {"\
By System:\n\
==========", 32},
  {"                     R E C E I V E D                       S E N T", 33},
  {"System      Files  KBytes     Time       ATP   Files  KBytes     Time       ATP\n", 34},
  {"\
\n\
Commands By System:\n\
===================", 35},
  {"\
\n\
By User:\n\
========", 36},
  {"                        S E N T", 37},
  {"User         Files   KBytes      Time\n", 38},
  {"\
\n\
By Day:\n\
=======", 39},
  {"Date       R E C E I V E D                     S E N T", 40},
  {"Mo/Dy  Files   KBytes       Time      Files   KBytes       Time\n", 41},
  {"   Date         R E C E I V E D                     S E N T", 42},
  {"Year/Mo/Dy  Files   KBytes       Time      Files   KBytes       Time\n", 43},
  {"\tReceived %10.0f (%4u Files) in %s: %4d cps\n", 44},
  {"\tSent     %10.0f (%4u Files) in %s: %4d cps\n", 45},
  {"\tReceived %10.0f bytes more than sent.\n", 46},
  {"\tSent     %10.0f bytes more than received.\n", 47},
  {"\tThe system has been connected for %s", 48},
  {" (%3.1f%% of dead time)\n", 49},
  {"\t\tentirely payed by the local system.\n", 50},
  {"\t\tof which %s (%5.1f%%) are payed by the local system\n", 51},
  {"\t\t     and %s (%5.1f%%) by this system.\n", 52},
  {"\t\tentirely payed by this system.\n", 53},
  {"Spent ", 54},
  {" in %s\n", 55},
  {"\tLast connection: %s\n", 56},
  {"\
\tThere have been %u call(s) (with an average of %.1f Ok/day):\n\
\t\t\t\t\t", 57},
  {"\
%3u OK (%3u IN, %3u OUT, %3u STOPPED)\n\
\t\t\t\t\t", 58},
  {"\
%3u OK (%3u IN, %3u OUT)\n\
\t\t\t\t\t", 59},
  {"%3u FAILED\n", 60},
  {"\tThere have been %u failed calls\n", 61},
  {"\
\n\
SUMMARY by System:\n\
==================", 62},
  {"|        |        R E C E I V E D       |            S E N T           |     |", 63},
  {"| SYSTEM | KiloBytes :   Time   : Files | KiloBytes :   Time   : Files | ATP |", 64},
  {"| TOTALS | %s :%10.10s: %5u | %s :%10.10s: %5u | ####|\n", 65},
  {"\
\n\
...and since %s\n", 66},
  {"\
\n\
\n\
Last 12 Months Activity\n\
=======================\n", 67},
  {"System", 68},
  {"TOTALS", 69},
  {"\
\n\
\n\
Global Hourly Activity (on a 20 minutes basis)\n\
==============================================", 70},
  {"\
\n\
\n\
Hourly Activity (on a 20 minutes basis) on %s\n\
======================================================\n", 71},
  {"\
Max Value :     %.0f bytes\n\
Scale     : '#'= %.0f bytes\n\
\n", 72},
  {"%s: system %s is unknown to me", 73},
  {"%s: user %s is unknown to me", 74},
  {"%s: port %s is unknown to me", 75},
  {"%s: Unknown report requested: I know about 'systems', 'users', 'ports' and 'daily' only", 76},
  {"Wrong number of Parameters: %s needs an UserName and its Alias", 77},
  {"Wrong number of Parameters: %s needs an UserName and its GlobAlias", 78},
  {"Out of memory!\n", 79},
};

int _msg_tbl_length = 79;
