/* this symbol was not defined in autoconf 1.5 standard acconfig.h. */
#undef CRAY_STACKSEG_END

/* define if your compiler handles __FUNCTION__ macro (gcc v2.xx) */
#undef HAVE_FUNCTION_MACRO

/* define if size_t is defined in stddef.h */
#undef HAVE_SIZE_T_IN_STDDEF_H

/* define if size_t is defined in sys/types.h */
#undef HAVE_SIZE_T_IN_TYPES_H

/* define if time_t is defined in time.h */
#undef HAVE_TIME_T_IN_TIME_H

/* define if time_t is defined in sys/types.h */
#undef HAVE_TIME_T_IN_TYPES_H

/*
 * UUCP SUPPORT:
 * Define TAYLOR_UUCP if you are using Taylor Uucp on your
 * machine.
 * Define HDB_UUCP if you are using HDB (aka BNU) Uucp on
 * your machine.
 * You can define 'em both, so you will choose the mode
 * with a command line option (--taylor-uucp || --hdb-uucp)
 */
#undef TAYLOR_UUCP
#undef HDB_UUCP

/*
 * If you are using Taylor Uucp compiled with SPOOLDIR_HDB=1, then define
 * TAYLOR_HDB, otherwise comment it out.
 */
#undef TAYLOR_HDB

/* Define to the name of the distribution.  */
#undef PACKAGE

/* Define to the version of the distribution.  */
#undef VERSION

/* Define if you have obstacks.  */
#undef HAVE_OBSTACK

/* Define if <stddef.h> defines ptrdiff_t.  */
#undef HAVE_PTRDIFF_T

/* Define if your locale.h file contains LC_MESSAGES.  */
#undef HAVE_LC_MESSAGES

/* Define if you have the parse_printf_format function.  */
#undef HAVE_PARSE_PRINTF_FORMAT

/* Define to 1 if NLS is requested.  */
#undef ENABLE_NLS

/* Define as 1 if you have catgets and don't want to use GNU gettext.  */
#undef HAVE_CATGETS

/* Define as 1 if you have gettext and don't want to use GNU gettext.  */
#undef HAVE_GETTEXT

/* Define as 1 if you have the stpcpy function.  */
#undef HAVE_STPCPY

/* Define as 1 if you want TCL support compiled in. */
#undef USE_TCL

/* Define as 1 if you want SoftChip BTree support compiled in. */
#undef USE_SOFTCHIP_BTREE

/* Define as 1 if you want GNU Readline support compiled in. */
#undef USE_READLINE

/* Define as 1 if you want Glob Aliases. */
#undef USE_GLOB_ALIASES
